
#include "config.h"

#if USE_FOST02

#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>

#include "RFM02.h"
#include "FOST02.h"

//-----------------------------------------------------------------------------

void fost02_write_start()
{
  FOST02_PORT |= (1<<FOST02_CLK);           // Clk auf H
  asm("nop");
  asm("nop");
  FOST02_DDR  |= (1<<FOST02_DATA);          // Data L durch Ausgang
  asm("nop");
  asm("nop");
  FOST02_PORT &= ~(1<<FOST02_CLK);          // Clk auf L
  asm("nop");
  asm("nop");
  asm("nop");
  asm("nop");
  FOST02_PORT |= (1<<FOST02_CLK);           // Clk auf H
  asm("nop");
  asm("nop");
  FOST02_DDR &= ~(1<<FOST02_DATA);          // Data H durch ext. PullUp
  asm("nop");
  asm("nop");
  FOST02_PORT &= ~(1<<FOST02_CLK);          // Clk auf L
  asm("nop");
  asm("nop");
}

//-----------------------------------------------------------------------------

void fost02_write_ack()
{
  asm("nop");
  asm("nop");
  FOST02_DDR  |= (1<<FOST02_DATA);          // Data L durch Ausgang
  asm("nop");
  asm("nop");
  FOST02_PORT |= (1<<FOST02_CLK);           // Clk auf H
  asm("nop");
  asm("nop");
  asm("nop");
  asm("nop");
  FOST02_PORT &= ~(1<<FOST02_CLK);          // Clk auf L
  asm("nop");
  asm("nop");
  FOST02_DDR &= ~(1<<FOST02_DATA);          // Data H durch ext. PullUp
  asm("nop");
  asm("nop");
}

//-----------------------------------------------------------------------------

void fost02_write_nack()
{
  asm("nop");
  asm("nop");
  FOST02_PORT |= (1<<FOST02_CLK);           // Clk auf H
  asm("nop");
  asm("nop");
  asm("nop");
  asm("nop");
  FOST02_PORT &= ~(1<<FOST02_CLK);          // Clk auf L
  asm("nop");
  asm("nop");
}

//-----------------------------------------------------------------------------

void fost02_read_ack()
{ 
  while (FOST02_PIN & (1<<FOST02_DATA))     // warten bis DATA auf L geht
    {
	  }

  FOST02_PORT |= (1<<FOST02_CLK);           // Clk auf H
  asm("nop");
  asm("nop");
  FOST02_PORT &= ~(1<<FOST02_CLK);          // Clk auf L
  asm("nop");
  asm("nop");
}

//-----------------------------------------------------------------------------

void fost02_wait_ready()
{

  while (FOST02_PIN & (1<<FOST02_DATA))     // auf DATA L vom Sensor warten
    {
	  }

}

//-----------------------------------------------------------------------------

void fost02_write_cmd(uint8_t byte)
{
  uint8_t i;

  for (i=0; i<8; i++)
    { 
      if (byte & 0x80)
	      {
		      FOST02_DDR &= ~(1<<FOST02_DATA);   // Data H durch ext. PullUp
        }
	    else
	      {
		      FOST02_DDR |= (1<<FOST02_DATA);    // Data L durch Ausgang
		    }   	   
      asm("nop");
      asm("nop");
      FOST02_PORT |= (1<<FOST02_CLK);       // Clk auf H
      asm("nop");
      asm("nop");
      FOST02_PORT &= ~(1<<FOST02_CLK);      // Clk auf L
      asm("nop");
      asm("nop");

      byte = (byte<<1);
    }
  FOST02_DDR &= ~(1<<FOST02_DATA);          // Data H durch ext. PullUp
}

//-----------------------------------------------------------------------------

uint8_t fost02_read_daten()
{
  uint8_t i, byte = 0;

  for (i=0; i<8; i++)
    { 
  	  byte = (byte<<1);
      FOST02_PORT |= (1<<FOST02_CLK);       // Clk auf H
      asm("nop");
      asm("nop");
      asm("nop");
      if (FOST02_PIN & (1<<FOST02_DATA))    // <> 0
        {
           byte = byte | 0x01;
        }
      FOST02_PORT &= ~(1<<FOST02_CLK);       // Clk auf L
      asm("nop");
      asm("nop");

	}  	  

  return byte;
}

//-----------------------------------------------------------------------------

void fost02_init()
{
  FOST02_PORT &= ~((1<<FOST02_CLK) | (1<<FOST02_DATA));         // CLK L,  DATA L, PullUp aus
  FOST02_DDR &= ~(1<<FOST02_DATA);                              // Data Eingang, ext. PullUp
  FOST02_DDR |= (1<<FOST02_CLK);                                // CLK Ausgang

  fost02_reset();
}

//-----------------------------------------------------------------------------

void fost02_reset()
{
  fost02_write_start();
  fost02_write_cmd(SOFT_RESET);
  fost02_read_ack();
  _delay_ms(20);
}

//-----------------------------------------------------------------------------

uint8_t fost02_read_status()
{  

  uint8_t byte;

  fost02_write_start();
  fost02_write_cmd(STATUS_READ);
  fost02_read_ack();
  byte = fost02_read_daten();
  fost02_write_nack();

  return byte;
}

//-----------------------------------------------------------------------------

uint16_t fost02_feuchte_temp(uint8_t command)
{
  uint16_t daten = 0;

  fost02_write_start();
  fost02_write_cmd(command);
  fost02_read_ack();

  fost02_wait_ready();                      // auf Ende der Messung warten

  daten = (fost02_read_daten()<<8);         // H-Byte
  fost02_write_ack();

  daten |= fost02_read_daten();             // L Byte
  fost02_write_nack();

  return daten;
}

//-----------------------------------------------------------------------------

void fost02_read()
{
  uint16_t daten, temp1;
  uint32_t temp2;
  int16_t wert;
  uint8_t i = 0, j = 0; 
  char text[6];                                              // Buffer zur Konvertierung
  
  FOST02_PORT &= ~(1<<FOST02_DATA);                          // Data PullUp aus / Ausgang L

  daten = fost02_feuchte_temp(TEMPERATUR_READ);
  wert = daten - 3962;        // berechnet Temperatur in Hundertstel Grad (Festkomma) bei 3,3V

  itoa(abs(wert),text,10);

  if (wert < 0)
    {
       buffer[SUB_OFFSET_1+j] = '-';
    }
  else
    {
      buffer[SUB_OFFSET_1+j]  = ' ';
    }
  
  j++;
  if (abs(wert) < 1000)                                  // nur 1 Stelle vor dem Komma
    {
       buffer[SUB_OFFSET_1+j] = ' ';
    }
  else
    {
      buffer[SUB_OFFSET_1+j]  = text[i];                      // Zehner
      i++;
    }
  j++;
  if (abs(wert) < 100)                                   // keine Stelle vor dem Komma
    {
       buffer[SUB_OFFSET_1+j] = '0';
       j++;
    }

   buffer[SUB_OFFSET_1+j]  = text[i];                         // Einer
   i++;
   j++;
   buffer[SUB_OFFSET_1+j] = ',';                              // Komma
   j++;
   buffer[SUB_OFFSET_1+j]  = text[i];                         // Zehntel
 
  daten = fost02_feuchte_temp(FEUCHTE_READ);
          // berechnet Feuchte in Hundertstel % (Festkomma) bei 3,3V
  temp1 = (1037UL * daten) / 256;                           // Feuchte * 1037 / 256 ->  * 4.05  (RF * 0,0405) erweitert mit 100  
  temp2 = (((uint32_t) daten * daten) / 256)  * 4698;          // Feuchte ^2 * 4698/256 -> * 18,35
  temp2 = (temp2 >> 16);                                    // (Feuchte ^2 * 4698/256 -> * 18,35) / 65536 -> * 0,00028
  wert = (temp1 - temp2) -400;                             // temp1 -temp2 - 400 (-4 erweitert um 100 wegen Rechnung 1)     
  itoa(wert,text,10);
  buffer[SUB_OFFSET_2+1] = text[0];
  buffer[SUB_OFFSET_2+2] = text[1];
  buffer[SUB_OFFSET_2+3] = ',';
  buffer[SUB_OFFSET_2+4] = text[2];
  buffer[SUB_OFFSET_2+5] = text[3];
}

//-----------------------------------------------------------------------------

#endif
