
#include <avr/io.h>

#include "config.h"
#include "rfm02.h"

uint8_t rfm02_send_cmd(uint16_t);
uint8_t rfm02_send_8bit(uint8_t);
void rfm02_send_byte(uint8_t);

char buffer[PAKET_LEN] = {(0x50 | SENSOR_NR),0x50,                  					// Sensornummer, Status			
                          (0x50 | SUB_TYP_1),0x35,0x20,0x20,0x20,0x20,0x20,		// Sub-Sensor 1
						              (0x50 | SUB_TYP_2),0x20,0x20,0x20,0x20,0x20,0x20,		// Sub-Sensor 2
						              (0x50 | SUB_TYP_3),0x20,0x20,0x20,0x20,0x20,0x20,		// Sub-Sensor 3
						               0x00};                                             // Checksumme

//-----------------------------------------------------------------------------------------

void rfm02_init()
{
#if USE_FOST02
  RFM02_PORT |= (1<<RFM02_CS);                     // SDI hat externen PullUp / CS Ausgang H
  RFM02_DDR  |= (1<<RFM02_CS) | (1<<RFM02_SCK);    // Ausgnge 
#else
  RFM02_PORT |= (1<<RFM02_IRQ) | (1<<RFM02_CS) | (1<<RFM02_SDI);    // PullUp ein / Ausgang H
  RFM02_DDR  |= (1<<RFM02_CS) | (1<<RFM02_SCK) | (1<<RFM02_SDI);    // Ausgnge 
#endif

  RFM02_DDR  &= ~(1<<RFM02_IRQ);                                    // Eingang

  rfm02_send_cmd(0xCC00);   // Status read

  rfm02_send_cmd(0xC0E0);   // Power Settings: Quarz ein, Synthesizer und PA ein bei Send-Command
  rfm02_send_cmd(0xC2A0);	  // low bat detector, TX Bit Sync, WakUp-Timer Calibrierung aus
  rfm02_send_cmd(0x8883);	  // Control: 1MHz Clock Out, 12,5pF Last Quarz, 120kHz Shift
  rfm02_send_cmd(0xA620);	  // Frequenz: 430MHz + (0,0025 * 0x620) -> 433,92MHz
  rfm02_send_cmd(0xB200);   // Power 2 * -3dBm -> -6dBm
  rfm02_send_cmd(0xD2C0);   // Set PLL: set 33%
  rfm02_send_cmd(0xC810);   // Baudrate: 344827 / 19200 = 17 - 1 -> 0x10                   // Status read
  rfm02_send_cmd(0xE500 + 200); // WakeUp-Timer 2^7 *234ms ~ 30s ???!! nicht 1ms sondern 10ms !!???
  rfm02_send_cmd(0xC002);   // alles aus, WakeUp-Timer ein

  rfm02_send_cmd(0xC440);   // sleep, noch 64 Takte
}

//-----------------------------------------------------------------------------------------

uint8_t rfm02_send_cmd(uint16_t command)
{
	uint8_t status;

  RFM02_PORT &= ~(1<<RFM02_CS);			      	     // CS auf L

	rfm02_send_8bit(command >> 8);                 // H-Byte senden
  
	status = rfm02_send_8bit(command & 0xFF);      // L-Byte senden

	RFM02_PORT |= (1<<RFM02_CS);					         // CS auf H

#if USE_FOST02
	RFM02_DDR &= ~(1<<RFM02_SDI);					         // SDI auf Eingang, H durch ext. PullUp
#endif

  return status;
}

//-----------------------------------------------------------------------------------------

uint8_t rfm02_send_8bit(uint8_t byte)
{
  uint8_t i;

	for (i=0; i<8; i++)
		{
			if (byte & 0x80)
			  {
#if USE_FOST02
          RFM02_DDR  &= ~(1<<RFM02_SDI);		// DATA auf H als Eingang durch PullUp
#else
			  	RFM02_PORT |= (1<<RFM02_SDI);		  // DATA auf H als Ausgang
#endif
			  }
			else
			  {
#if USE_FOST02
				RFM02_DDR |= (1<<RFM02_SDI);		    // DATA auf L als Ausgang
#else		
				RFM02_PORT &= ~(1<<RFM02_SDI);		  // DATA auf L
#endif
			  }
			asm("nop");
		  	RFM02_PORT |= (1<<RFM02_SCK);			  // CLK auf H
			asm("nop");
			asm("nop");

			byte = (byte << 1);			            	// nchstes Bit nach oben

			if (RFM02_PIN & (1<<RFM02_IRQ))       // unteres Bit Status einlesen, H?
			  {
			  	byte |= 0x01;                     // DATA war H
        }
			else
			  {
			  	byte &= ~(0x01);               		// DATA war L
			  }
			asm("nop");
		  	RFM02_PORT &= ~(1<<RFM02_SCK);			// CLK auf L
		}
  return byte;
}

//-----------------------------------------------------------------------------------------

void rfm02_send_byte(uint8_t byte)
{
  uint8_t i;

	for (i=0; i<8; i++)
		{
      while (!(RFM02_PIN & (1<<RFM02_IRQ))) // warten bis IRQ L/H
        {
        }

      while (RFM02_PIN & (1<<RFM02_IRQ))    // warten bis IRQ Impuls zuende H/L
        {
        }

			if (byte & 0x80)
			  {
#if USE_FOST02
          RFM02_DDR  &= ~(1<<RFM02_SDI);		// DATA auf H als Eingang durch PullUp
#else
			  	RFM02_PORT |= (1<<RFM02_SDI);		  // DATA auf H als Ausgang
#endif
			  }
			else
			  {
#if USE_FOST02
				RFM02_DDR |= (1<<RFM02_SDI);		    // DATA auf L als Ausgang
#else		
				RFM02_PORT &= ~(1<<RFM02_SDI);		  // DATA auf L
#endif
			  }
      byte = (byte << 1);                   // Bit 7 rausschieben
		}
}

//-----------------------------------------------------------------------------------------

void rfm02_send_data()
{
  uint8_t i;

  rfm02_send_cmd(0xC0E0);                   // einschalten

  RFM02_PORT &= ~(1<<RFM02_CS);			      	// CS auf L

  rfm02_send_8bit(0xC6);                    // Kommando Daten senden

  rfm02_send_byte(0xAA);                    // Sync
  rfm02_send_byte(0xAA);
  rfm02_send_byte(0xAA);
  rfm02_send_byte(0x2D);                    // Magic
  rfm02_send_byte(0xD4);
  
  for (i=0; i<PAKET_LEN; i++)
    {
      rfm02_send_byte(buffer[i]);      
    }
  while (!(RFM02_PIN & (1<<RFM02_IRQ)))     // auf Ende sendes letztes Byte warten -> L/H
    {
    }
  while (RFM02_PIN & (1<<RFM02_IRQ))        // erledigt, wenn IRQ wieder -> H/L
    {
    }
  while (!(RFM02_PIN & (1<<RFM02_IRQ)))     // auf Ende Power Off warten -> L/H
    {
    }

	RFM02_PORT |= (1<<RFM02_CS);					    // CS auf H

#if USE_FOST02
	RFM02_DDR &= ~(1<<RFM02_SDI);					    // SDI auf Eingang, H durch ext. PullUp
#endif

  rfm02_send_cmd(0xC002); 						      // alles aus, WakeUp-Timer ein
  rfm02_send_cmd(0xC440);     						  // sleep, noch 64 Takte

}

//-----------------------------------------------------------------------------------------

uint8_t rfm02_check_batt()
{
  uint8_t i;
  uint8_t status;

  rfm02_send_cmd(0xC0E4);                   // einschalten

  for (i=4; i<20; i++)                      // von 2,2V + 4*0,1V = 2,6V bis 2.2V + 19*0,1V = 4,1V
    {
      rfm02_send_cmd(0xC2A0 | i);           // Schwellwert setzen
      status = rfm02_send_cmd(0xCC00);      // Status lesen
      if (status & 0x0002)                  // LowPoerDetektor auf 1?
        {
          break;                            // Schleife beenden, i enthlt aktuellen Offset
        }
    }
  rfm02_send_cmd(0xC002); 						      // alles aus, WakeUp-Timer ein
  rfm02_send_cmd(0xC440);     						  // sleep, noch 64 Takte
  
  return i;
}

//-----------------------------------------------------------------------------

