
#include "config.h"

#if USE_TMP36S

#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>

#include "RFM02.h"
#include "TMP36S.h"

void convert_tmp36s(uint16_t);

void tmp36_read()
{

  uint8_t i;
  uint16_t daten = 0;

  TMP36_PORT &= ~(1<<TMP36_ADC);                              // PullUp aus
  TMP36_DDR  &= ~(1<<TMP36_ADC);                              // Eingang     
  TMP36_PORT |= (1<<TMP36_OFF);                               // TMP36S ein

  _delay_ms(20);

  ADMUX  = (1<<REFS1) | TMP36_MUX;              			  // 1,1V Refs, ADC3
  ADCSRA = (1<<ADEN) | (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0);  // ADC ein, Prescale 64 -> 125kHz

  for (i=0; i<8; i++)
    {
      ADCSRA |= (1<<ADSC);                                    //  Start

      while (ADCSRA & (1<<ADSC))                              // warten bis fertig
	      {
        }
      daten += ADC;                                           // Daten dazu 
    } 

  ADCSRA &= ~(1<<ADEN);                                       // ADC aus
  TMP36_PORT &= ~(1<<TMP36_OFF);                              // TMP36 aus

  convert_tmp36s(daten >> 3);								  // durch 8 als Ergebnis, umrechnen und in buffer kopieren
}

//-----------------------------------------------------------------------------

void convert_tmp36s(uint16_t daten)
{
  int32_t temperatur;
  uint8_t i = 0, j = 0;
  char text[6];                                              // Buffer zur Konvertierung

  temperatur = (AREF_AKT * 256L)/1024 * daten;               // Umrechnung Spannung und Referenz 
  temperatur = (temperatur / 256) - 500;

  itoa(abs(temperatur),text,10);

  if (temperatur < 0)
    {
       buffer[SUB_OFFSET_1+j] = '-';
    }
  else
    {
      buffer[SUB_OFFSET_1+j]  = ' ';
    }
  
  j++;
  if (abs(temperatur) < 100)                                  // nur 1 Stelle vor dem Komma
    {
       buffer[SUB_OFFSET_1+j] = ' ';
    }
  else
    {
      buffer[SUB_OFFSET_1+j]  = text[i];                      // Zehner
      i++;
    }
  j++;
  if (abs(temperatur) < 10)                                   // keine Stelle vor dem Komma
    {
       buffer[SUB_OFFSET_1+j] = '0';
       j++;
    }

   buffer[SUB_OFFSET_1+j]  = text[i];                         // Einer
   i++;
   j++;
   buffer[SUB_OFFSET_1+j] = ',';                              // Komma
   j++;
   buffer[SUB_OFFSET_1+j]  = text[i];                         // Zehntel
 
}

#endif
