#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <util/delay.h>

#include "config.h"
#include "RFM02.h"
#include "TMP36S.h"
#include "TINY.h"
#include "SWITCH.h"
#include "FOST02.h"

void checksum(void);

//-----------------------------------------------------------------------------

int main()
{
  uint16_t i;

  for (i=0; i<50; i++)
    {
      _delay_ms(20);
    }

  PCMSK = (1<<RFM02_IRQ);							    // Pin Change IRQ
  set_sleep_mode(SLEEP_MODE_PWR_DOWN);
  sleep_enable();

  rfm02_init();

#if USE_FOST02
  fost02_init();				
#endif

#if USE_SWITCH
  switch_init();				
#endif

  sei();

  while(1)
    {

#if USE_TMP36S
      tmp36_read();									      // Sensordaten lesen, umrechnen und in den buffer kopieren				
#endif

#if USE_TINY
      tiny_read();									      // Sensordaten lesen, umrechnen und in den buffer kopieren				
#endif

#if USE_FOST02
      fost02_read();								      // Sensordaten lesen, umrechnen und in den buffer kopieren				
#endif

#if USE_SWITCH
      switch_read();								      // Sensordaten lesen, umrechnen und in den buffer kopieren				
#endif

      checksum();

	    rfm02_send_data();

      GIFR = (1<<PCIF);					          // evtl. noch anstehenden IRQ lschen
      GIMSK = (1<<PCIE);					        // Pin-Change-IRQ an

      sleep_cpu();                        // und schlafen schicken
/*
      for (i=0; i<200; i++)
        {
          _delay_ms(20);
        }
*/
    }
}

//-----------------------------------------------------------------------------

void checksum()
{
  uint8_t i, byte = 0;

  for (i=0; i<(PAKET_LEN-1); i++)					// XOR Prfsumme
    {
      byte = byte ^ buffer[i];     
    }
  buffer[PAKET_LEN-1] = byte;
}

//-----------------------------------------------------------------------------

ISR(PCINT0_vect)
{
  GIMSK = 0;						                  // Pin-Change-IRQ aus
}

//-----------------------------------------------------------------------------

